if select(6, GetAddOnInfo("PitBull4_" .. (debugstack():match("[o%.][d%.][u%.]les\\(.-)\\") or ""))) ~= "MISSING" then return end

local PitBull4 = PitBull4
if (not PitBull4) then
	error("PitBull4_RangeFader2 requires PitBull4")
end

-- cache
local L = LibStub("AceLocale-3.0"):GetLocale("PitBull4_RangeFader2")
local PitBull4_RangeFader2 = PitBull4:NewModule("RangeFader2", "AceTimer-3.0")
local rc = LibStub("LibRangeCheck-2.0")
local wipe = wipe
local UnitCanAttack = UnitCanAttack
local UnitCanAssist = UnitCanAssist
local UnitIsVisible = UnitIsVisible
local UnitIsDeadOrGhost = UnitIsDeadOrGhost

local UpdateDelay = 0.7

local checkers = {}

local function yes() return true end

local function getCheckers(db)
    local c = checkers[db]
    if not c then
        c = {}
        checkers[db] = c
        local checker, range = rc:GetHarmMaxChecker(db.enemyFadeRange)
        if not checker then
            c.harmLongChecker = yes
        elseif db.enemyFadeRange == 100 and range < 100 then
            c.harmLongChecker = UnitIsVisible
        else
            c.harmLongChecker = checker
        end
        if db.enemyHalfFadeRange >= db.enemyFadeRange then
            c.harmShortChecker = yes
        else
            checker, range = rc:GetHarmMaxChecker(db.enemyHalfFadeRange)
            if not checker then
                c.harmShortChecker = yes
            else
                c.harmShortChecker = checker
            end
        end

        checker, range = rc:GetFriendMaxChecker(db.friendFadeRange)
        if not checker then
            c.friendChecker = yes
        elseif db.friendFadeRange == 100 and range < 100 then
            c.friendChecker = UnitIsVisible
        else
            c.friendChecker = checker
        end

        checker, range = rc:GetMiscMaxChecker(db.otherFadeRange)
        if not checker then
            c.miscChecker = yes
        elseif db.otherFadeRange == 100 and range < 100 then
            c.miscChecker = UnitIsVisible
        else
            c.miscChecker = checker
        end
    end

    return c
end

PitBull4_RangeFader2:SetModuleType("fader")
PitBull4_RangeFader2:SetName(L["Range fader 2"])
PitBull4_RangeFader2:SetDescription(L["Make the unit frame fade if out of range."])
PitBull4_RangeFader2:SetDefaults({
    enabled = false,
    oorOpacity = 0.6,
    enemyFadeRange = 40,
    enemyHalfFadeRange = 30,
    friendFadeRange = 40,
    otherFadeRange = 30,
})

function PitBull4_RangeFader2:GetOpacity(frame)
    local unit = frame.unit
    local db = self:GetLayoutDB(frame)
    local c = getCheckers(db)
    if UnitIsDeadOrGhost(unit) then
        if c.miscChecker(unit) then
            return 1
        else
            return db.oorOpacity
        end
    elseif UnitCanAssist("player", unit) then
        if c.friendChecker(unit) then
            return 1
        else
            return db.oorOpacity
        end
    elseif UnitCanAttack("player", unit) then
        if not c.harmLongChecker(unit) then
            return db.oorOpacity
        elseif not c.harmShortChecker(unit) then
            return (db.oorOpacity + frame.layout_db.opacity_max) / 2
        else
            return 1
        end
    else
        if c.miscChecker(unit) then
            return 1
        else
            return db.oorOpacity
        end
    end
end

function PitBull4_RangeFader2:OnEnable()
    self:ScheduleRepeatingTimer("UpdateNonWacky", UpdateDelay)
    rc.RegisterCallback(self, rc.CHECKERS_CHANGED, "CHECKERS_CHANGED")
end

function PitBull4_RangeFader2:OnDisable()
    self:UnregisterAllEvents()
    rc.UnregisterAllCallbacks(self)
end

function PitBull4_RangeFader2:CHECKERS_CHANGED()
    wipe(checkers)
end

PitBull4_RangeFader2:SetLayoutOptionsFunction(function(self)
	return
        'oorOpacity', {
            type = 'range',
            name = L["Out-of-range opacity"],
            desc = L["The opacity to display if the player is out of range of the unit."],
            min = 0,
            max = 1,
            isPercent = true,
            get = function(info)
                local db = PitBull4.Options.GetLayoutDB(self)
                return db.oorOpacity
            end,
            set = function(info, value)
                local db = PitBull4.Options.GetLayoutDB(self)
                db.oorOpacity = value

                PitBull4.Options.UpdateFrames()
                PitBull4:RecheckAllOpacities()
            end,
            step = 0.01,
            bigStep = 0.05,
        },
        'enemyFadeRange', {
            type = 'range',
            name = L["Enemy fade range"],
            min = 0,
            max = 100,
            get = function(info)
                local db = PitBull4.Options.GetLayoutDB(self)
                return db[info[#info]]
            end,
            set = function(info, value)
                local db = PitBull4.Options.GetLayoutDB(self)
                db[info[#info]] = value
                wipe(checkers)
            end,
            step = 1,
            bigStep = 5,
        },
        'enemyHalfFadeRange', {
            type = 'range',
            name = L["Enemy half-fade range"],
            min = 0,
            max = 100,
            get = function(info)
                local db = PitBull4.Options.GetLayoutDB(self)
                return db[info[#info]]
            end,
            set = function(info, value)
                local db = PitBull4.Options.GetLayoutDB(self)
                db[info[#info]] = value
                wipe(checkers)
            end,
            step = 1,
            bigStep = 5,
        },
        'friendFadeRange', {
            type = 'range',
            name = L["Friend fade range"],
            min = 0,
            max = 100,
            get = function(info)
                local db = PitBull4.Options.GetLayoutDB(self)
                return db[info[#info]]
            end,
            set = function(info, value)
                local db = PitBull4.Options.GetLayoutDB(self)
                db[info[#info]] = value
                wipe(checkers)
            end,
            step = 1,
            bigStep = 5,
        },
        'otherFadeRange', {
            type = 'range',
            name = L["Other fade range"],
            min = 0,
            max = 100,
            get = function(info)
                local db = PitBull4.Options.GetLayoutDB(self)
                return db[info[#info]]
            end,
            set = function(info, value)
                local db = PitBull4.Options.GetLayoutDB(self)
                db[info[#info]] = value
                wipe(checkers)
            end,
            step = 1,
            bigStep = 5,
        }
end)

